package sun.rmi.server;

import java.rmi.UnmarshalException;
import java.io.ObjectInput;
import java.io.IOException;
import java.io.ObjectOutput;
import java.rmi.server.RemoteCall;
import java.rmi.server.Operation;
import java.rmi.server.RemoteObject;
import java.rmi.activation.ActivationException;
import java.rmi.activation.ActivateFailedException;
import java.rmi.activation.UnknownObjectException;
import java.lang.reflect.Proxy;
import java.rmi.server.RemoteObjectInvocationHandler;
import java.rmi.server.RemoteStub;
import java.rmi.RemoteException;
import java.rmi.ServerException;
import java.rmi.ServerError;
import java.rmi.MarshalException;
import java.rmi.ConnectIOException;
import java.rmi.UnknownHostException;
import java.rmi.ConnectException;
import java.rmi.NoSuchObjectException;
import java.lang.reflect.Method;
import java.net.MalformedURLException;
import java.rmi.StubNotFoundException;
import java.rmi.server.RMIClassLoader;
import java.rmi.Remote;
import java.rmi.activation.ActivationDesc;
import java.rmi.activation.ActivationID;
import java.rmi.server.RemoteRef;

public class ActivatableRef implements RemoteRef
{
    private static final long serialVersionUID = 7579060052569229166L;
    protected ActivationID id;
    protected RemoteRef ref;
    transient boolean force;
    private static final int MAX_RETRIES = 3;
    private static final String versionComplaint = "activation requires 1.2 stubs";
    
    public ActivatableRef() {
        this.force = false;
    }
    
    public ActivatableRef(ActivationID id, RemoteRef ref) {
        this.force = false;
        this.id = id;
        this.ref = ref;
    }
    
    public static Remote getStub(ActivationDesc activationDesc, ActivationID activationID) throws StubNotFoundException {
        String className = activationDesc.getClassName();
        try {
            return Util.createProxy(RMIClassLoader.loadClass(activationDesc.getLocation(), className), new ActivatableRef(activationID, null), false);
        }
        catch (IllegalArgumentException ex) {
            throw new StubNotFoundException("class implements an illegal remote interface", ex);
        }
        catch (ClassNotFoundException ex2) {
            throw new StubNotFoundException("unable to load class: " + className, ex2);
        }
        catch (MalformedURLException ex3) {
            throw new StubNotFoundException("malformed URL", ex3);
        }
    }
    
    @Override
    public Object invoke(Remote remote, Method method, Object[] array, long n) throws Exception {
        boolean n2 = false;
        RemoteException ex = null;
        RemoteRef remoteRef;
        synchronized (this) {
            if (this.ref == null) {
                remoteRef = this.activate((boolean)n2);
                n2 = true;
            }
            else {
                remoteRef = this.ref;
            }
        }
        for (int i = 3; i > 0; --i) {
            try {
                return remoteRef.invoke(remote, method, array, n);
            }
            catch (NoSuchObjectException ex2) {
                ex = ex2;
            }
            catch (ConnectException ex3) {
                ex = ex3;
            }
            catch (UnknownHostException ex4) {
                ex = ex4;
            }
            catch (ConnectIOException ex5) {
                ex = ex5;
            }
            catch (MarshalException ex6) {
                throw ex6;
            }
            catch (ServerError serverError) {
                throw serverError;
            }
            catch (ServerException ex7) {
                throw ex7;
            }
            catch (RemoteException ex8) {
                synchronized (this) {
                    if (remoteRef == this.ref) {
                        this.ref = null;
                    }
                }
                throw ex8;
            }
            if (i > 1) {
                synchronized (this) {
                    if (remoteRef.remoteEquals(this.ref) || this.ref == null) {
                        RemoteRef remoteRef2 = this.activate((boolean)n2);
                        if (remoteRef2.remoteEquals(remoteRef) && ex instanceof NoSuchObjectException && n2 == 0) {
                            remoteRef2 = this.activate(true);
                        }
                        remoteRef = remoteRef2;
                        n2 = true;
                    }
                    else {
                        remoteRef = this.ref;
                        n2 = false;
                    }
                }
            }
        }
        throw ex;
    }
    
    private synchronized RemoteRef getRef() throws RemoteException {
        if (this.ref == null) {
            this.ref = this.activate(false);
        }
        return this.ref;
    }
    
    private RemoteRef activate(boolean b) throws RemoteException {
        assert !(!Thread.holdsLock(this));
        this.ref = null;
        try {
            Remote activate = this.id.activate(b);
            ActivatableRef activatableRef;
            if ((activate instanceof RemoteStub)) {
                activatableRef = (ActivatableRef)((RemoteStub)activate).getRef();
            }
            else {
                activatableRef = (ActivatableRef)((RemoteObjectInvocationHandler)Proxy.getInvocationHandler(activate)).getRef();
            }
            return this.ref = activatableRef.ref;
        }
        catch (ConnectException ex) {
            throw new ConnectException("activation failed", ex);
        }
        catch (RemoteException ex2) {
            throw new ConnectIOException("activation failed", ex2);
        }
        catch (UnknownObjectException ex3) {
            throw new NoSuchObjectException("object not registered");
        }
        catch (ActivationException ex4) {
            throw new ActivateFailedException("activation failed", ex4);
        }
    }
    
    @Override
    public synchronized RemoteCall newCall(RemoteObject remoteObject, Operation[] array, int n, long n2) throws RemoteException {
        throw new UnsupportedOperationException("activation requires 1.2 stubs");
    }
    
    @Override
    public void invoke(RemoteCall remoteCall) throws Exception {
        throw new UnsupportedOperationException("activation requires 1.2 stubs");
    }
    
    @Override
    public void done(RemoteCall remoteCall) throws RemoteException {
        throw new UnsupportedOperationException("activation requires 1.2 stubs");
    }
    
    @Override
    public String getRefClass(ObjectOutput objectOutput) {
        return "ActivatableRef";
    }
    
    @Override
    public void writeExternal(ObjectOutput objectOutput) throws IOException {
        RemoteRef ref = this.ref;
        objectOutput.writeObject(this.id);
        if (ref == null) {
            objectOutput.writeUTF("");
        }
        else {
            objectOutput.writeUTF(ref.getRefClass(objectOutput));
            ref.writeExternal(objectOutput);
        }
    }
    
    @Override
    public void readExternal(ObjectInput objectInput) throws IOException, ClassNotFoundException {
        this.id = (ActivationID)objectInput.readObject();
        this.ref = null;
        String utf = objectInput.readUTF();
        if (utf.equals("")) {
            return;
        }
        try {
            (this.ref = (RemoteRef)Class.forName("sun.rmi.server." + utf).newInstance()).readExternal(objectInput);
        }
        catch (InstantiationException ex) {
            throw new UnmarshalException("Unable to create remote reference", ex);
        }
        catch (IllegalAccessException ex2) {
            throw new UnmarshalException("Illegal access creating remote reference");
        }
        return;
    }
    
    @Override
    public String remoteToString() {
        return Util.getUnqualifiedName(this.getClass()) + " [remoteRef: " + this.ref + "]";
    }
    
    @Override
    public int remoteHashCode() {
        return this.id.hashCode();
    }
    
    @Override
    public boolean remoteEquals(RemoteRef remoteRef) {
        return remoteRef instanceof ActivatableRef && this.id.equals(((ActivatableRef)remoteRef).id);
    }
}
